/*
 * Microstepping demo
 *
 * This requires that M0, M1 be connected in addition to STEP,DIR
 *
 * Copyright (C)2015 Laurentiu Badea
 *
 * This file may be redistributed under the terms of the MIT license.
 * A copy of this license has been included with this distribution in the file LICENSE.
 */

#include <Arduino.h>
#include <Servo.h>
#include "Messenger.h"

//Define the Number of Filters on your Filter Wheel
#define NRFILTERS 3

// All the wires needed for full functionality
#define PINSERVO 9
#define ANGLE_CLEAR 105
#define ANGLE_NEON 30
#define ANGLE_FLAT 30
#define ANGLE_CLOSE 30

char CurrentPosition;
Servo monServo;
Messenger message = Messenger();

byte Compteur = 0;  //Compteur pour interruption
byte Compteur2 = 0; // Compteur pour led connected mode auto 
byte Compteur3=0; // compteur pour led non connected mode auto 


const int pin_LedR = 4; // broche 4 du micro-contrôleur couleur rouge de la LED
const int pin_LedV = 5 ;  // broche 5 du micro-contrôleur couleur vert de la LED
const int pin_LedB = 6;  // broche 6 du micro-contrôleur couleur bleu de la LED
const int pin_LedAlim = 7; // broche 7 du micro-contrôleur continu Alimentation clignotant mode connecté
const int pin_Flat = 8; // broche 8 du micro-contrôleur de la lampe du flat
const int pin_BP = 3; // broche 3 du micro-contrôleur se nomme maintenant : Bouton poussoir
const byte pin_Led = 13; // alimentation HT
const byte pin_Led_Bar=12; // alimentation HT opposé 
const byte pin_temp = A0 ; // capteur de temperature
const byte pin_connected = 11 ; // Mode manuel ou connecté   5V : connecte    0V:manuel  

int state = LOW; // variable d'état de la LED (précédée de volatile !!)
int buttonState;
int laststate = LOW;
int reading;
int CompteurState = 0; // compteur d'état 0:
int CompteurStateOld = 0; // ancienn valeur de CompteurState
long lastDebounceTime = 0; //
long debounceDelay = 50; // timer pour eviter le rebond du BP
long  temp;  /// lecture de l'entrée analogique de la temperature
double temperature ; //calcul de la temperature
bool initialisation = true; // detection du 1er appel
bool Connected = false; // etat du mode connecté ou manu  

void messageCompleted() {
  interpretCommand(&message);
}


void setup() {

  // Initialise le Timer 2 pour déclencher les interruptions à intervalle régulier
  TCCR2A = 0; //default
  TCCR2B = 0b00000110; // clk/256 est incrémenté toutes les 16uS
  TIMSK2 = 0b00000001; // TOIE2
  sei();               // autorise les interruptions


  //contenu de l'initialisation
  pinMode(pin_LedR, OUTPUT); //L1 est une broche de sortie*
  pinMode(pin_LedV, OUTPUT); //L1 est une broche de sortie
  pinMode(pin_LedB, OUTPUT); //L1 est une broche de sortie
  pinMode(pin_LedR, OUTPUT); //L1 est une broche de sortie
  digitalWrite(pin_Led_Bar,HIGH);
  pinMode(pin_Led, OUTPUT); //L1 est une broche de sortie
  digitalWrite(pin_Led,HIGH);
  pinMode(pin_LedAlim, OUTPUT); //L1 est une broche de sortie
  pinMode(pin_Flat, OUTPUT); // Led mode manu ou connecté
  pinMode(pin_BP, INPUT); // BP est une broche d'entree
  //attachInterrupt(digitalPinToInterrupt(BP), blink, CHANGE);

  Serial.flush();
  Serial.begin(9600);  // Baud rate, make sure this is the same as ASCOM driver
  monServo.attach(PINSERVO,0,1000);
  monServo.write(ANGLE_CLEAR);
  CurrentPosition = '0';
  lastDebounceTime = millis();
  digitalWrite(pin_LedAlim, HIGH); // LED alimentation toujours allumée
  Serial.println("INITIALISATION");
  message.attach(messageCompleted);

}

void loop() {

  String ASCOMcmd;
  
  // MEsure temperature
  temp = analogRead(pin_temp);
  temperature = (double) temp * (5 / 10.24);

  // Mode connecté ou manuel
  if (!digitalRead(pin_connected) && Connected) {
      Connected = false;
      Serial.println("DISCONNECT");
  }
 
  // bouton poussoir
  if ((millis() - lastDebounceTime) > debounceDelay && !digitalRead(pin_connected))
  {
    reading = digitalRead(pin_BP);

    if (reading == 1 && laststate == 0)
    {
      CompteurState++;
      state = reading;
      //Serial.print(CompteurState);
      if (CompteurState == 4) {
        CompteurState = 0;
      }
    }
    laststate = reading;
    lastDebounceTime = millis();
  }
  // Position 0  Clear
  if (CompteurState == 0 && (CompteurStateOld != 0)) {
    digitalWrite(pin_LedR, LOW);
    digitalWrite(pin_LedV, LOW);
    digitalWrite(pin_LedB, LOW);
    digitalWrite(pin_Flat,LOW);
    MoveFilter('0');
    CompteurStateOld = 0;
  }
  // Position 1 calibRATION NEON
  if (CompteurState == 1 && CompteurStateOld != 1) {
    digitalWrite(pin_LedR, HIGH);
    digitalWrite(pin_LedV, LOW);
    digitalWrite(pin_LedB, LOW);
    digitalWrite(pin_Flat,LOW);
    MoveFilter('1');
    CompteurStateOld = 1;
  }
  // Position 2 flat tungstene
  if (CompteurState == 2 && CompteurStateOld != 2) {
    digitalWrite(pin_LedR, LOW);
    digitalWrite(pin_LedV, HIGH);
    digitalWrite(pin_LedB, LOW);
    digitalWrite(pin_Flat,HIGH);
    MoveFilter('2');
    CompteurStateOld = 2;
  }
  // Position fermée seulement
  if (CompteurState == 3 && CompteurStateOld != 3) {
    digitalWrite(pin_LedR, LOW);
    digitalWrite(pin_LedV, LOW);
    digitalWrite(pin_LedB, HIGH);
    digitalWrite(pin_Flat,LOW);
    MoveFilter('3');
    CompteurStateOld = 3;
  }
  // dephasage de la RELCO  
  if (CompteurState==1) {digitalWrite(pin_Led_Bar,!digitalRead(pin_Led));} else {digitalWrite(pin_Led_Bar,LOW);digitalWrite(pin_Led,LOW);} ;
 
if (Serial.available()) message.process(Serial.read());

  if (Serial.available() > 0 && digitalRead(pin_connected)) {
    ASCOMcmd = Serial.readStringUntil('#');  // Terminator so arduino knows when the message ends
    
    if (ASCOMcmd == "GETFILTER" ) {
      Serial.print(CurrentPosition);
      Serial.println("#");  // Similarly, so ASCOM knows
    }
    // Get the int value of the Filter Postition. cmd is FILTER0 - FILTERx. Last position in cmd is the number of the required Filter.
    //   So lets get that number
    //int NextPosition = (ASCOMcmd.substring(ASCOMcmd.length()-1)).toInt();
    if (((ASCOMcmd == "FILTER0" || ASCOMcmd == "FILTER1" || ASCOMcmd == "FILTER2" || ASCOMcmd == "FILTER3"))) {
      char stTemp[2];
      sprintf(stTemp, "%c", ASCOMcmd[6]);
      actionEtat(atoi(stTemp));
      initialisation = false;
    }
    if (ASCOMcmd == "GETTEMP") {
      GetTemperature();
    }
    if (ASCOMcmd == "CONNECTED" && digitalRead(pin_connected)) {
      Connected = true;
      //Serial.print(CurrentPosition);
      Serial.println("#");
    }
    if (ASCOMcmd == "DISCONNECTED") {
      Connected = false;
      Serial.println("#");
    }

  }
  //digitalWrite(pin_LedConnected,HIGH);
}

void MoveFilter(char NextPosition) {

  if (NextPosition == '0')
  {
    if (initialisation == true) {
      CompteurStateOld = '1'; // gere le premier appel
    }
    monServo.write(ANGLE_CLEAR);
        
  }
  else if (NextPosition == '1')
  {
   monServo.write(ANGLE_NEON);
  }
  else if (NextPosition == '2')
  {
    monServo.write(ANGLE_FLAT);
  }
  else if (NextPosition == '3')
  {
    monServo.write(ANGLE_CLOSE);
  }
  CurrentPosition = NextPosition;    // Note that the position is always 0 when powered on so have filters oriented properly
  Serial.print(CurrentPosition);
  Serial.println("#");  // Similarly, so ASCOM knows

}
// routine d'interruption du timer
ISR (TIMER2_OVF_vect)
{
  // TCNT2=6
  // 256-6 --> 250X16uS = 4mS
  // Recharge le timer pour que la prochaine interruption se déclenche dans 4mS
  TCNT2 =200;
  if (CompteurState == 1 && CompteurStateOld == 1)
  {
    if (Compteur++ == 1) {
      //250*4mS = 1S - la Led est allumée 1 S et éteinte 1 S
      Compteur = 0;
      bool etat = digitalRead(pin_Led);
      digitalWrite (pin_Led,!etat);
      
    }
       
  }
  if (Connected && digitalRead(pin_connected))    // LED mode auto ou manu 
  {
    if (Compteur2++ == 50) {  
      //250*4mS = 1S - la Led est allumée 1 S et éteinte 1 S
      Compteur2 = 0;
      digitalWrite (pin_LedAlim, !digitalRead(pin_LedAlim));
    }
  }
  else if ( !Connected && digitalRead(pin_connected))
  {
    if (Compteur3++==200){
      Compteur3=0;
      digitalWrite (pin_LedAlim,!digitalRead(pin_LedAlim));
      }
  }
  else
  {
    digitalWrite (pin_LedAlim,HIGH);
  }
}



void GetTemperature() {
  Serial.println("Temp : " + String(temperature));
}
void actionEtat( int etat)
{
  CompteurState = etat;
  if (initialisation == true && CompteurState == 0) {
    CompteurStateOld = 1;
  }

}
void interpretCommand(Messenger *message)
{
  message->readChar(); // Reads ":"
  char command = message->readChar(); // Read the command
  
  switch(command){
    case 'O':  // position Clear 
	if (digitalRead(pin_connected))
        {
          CompteurState = 0;
          Serial.println("FILTER 0");
        }
	break;
    case 'N' : // position neon
        if (digitalRead(pin_connected))
        {  
          CompteurState = 1 ;	
	  Serial.println("FILTER 1");
	}
        break;
    case 'F':  // position flat 
	if (digitalRead(pin_connected))
        {
          CompteurState = 2;
          Serial.println("FILTER 2");
	}
        break;
    case 'P': // position close
	if (digitalRead(pin_connected))
        {
          CompteurState = 3;
          Serial.println("FILTER 3");
	}
        break;
    case 'C':   // Connected 
        if (digitalRead(pin_connected))
        {
          Connected=true;
          Serial.println("CONNECTED");
        }
        else
        { 
          Serial.println("DISCONNECT");
        }
	break;
    case 'D':   // disconnected 
	if (digitalRead(pin_connected))
        {
          Connected=false;
          Serial.println("DISCONNECT");
	}
        break;
    case 'R' :  // get temperature 
        if (digitalRead(pin_connected))
       {
          Serial.print("TEMP ");
          Serial.println(temperature);
        }
        break;
  }
}



